from .util import *


class __THMLeaderboard(object):
    def get_leaderboard(self) -> list:
        """
        Returns the top 50 users from the all-time leaderboard

        :return: List containing top 50 users
        """

        return http_get(self.session, '/api/leaderboards')['topUsers']

    def get_monthly_leaderboard(self) -> list:
        """
        Returns the top 50 users from the monthly leaderboard

        :return: List containing top 50 users
        """

        return http_get(self.session, '/api/leaderboards')['topUsersMonthly']



from .util import *


class __THMInstance(object):
    def instance_running_instances(self) -> list:
        """
        AUTHENTICATED
        Gets the list of running instances

        :return: List of running instances
        """
        if not self.authenticated:
            raise Exception("Not authenticated")

        return http_get(self.session, '/api/running-instances')

    def instance_deploy(self, room_code, vm_id) -> dict:
        """
        AUTHENTICATED
        Deploys the selected VM in the room

        :param room_code: Room code to renew the VM in
        :param vm_id: VM id
        """
        if not self.authenticated:
            raise Exception("Not authenticated")

        csrf_token = fetch_pattern(self.session, f'/room/{room_code}', 'csrf-script')

        return http_post(
            self.session,
            '/deploy',
            data={'roomCode': room_code, 'id': vm_id},
            headers={'csrf-token': csrf_token},
            has_success=True
        )

    def instance_renew(self, room_code) -> float:
        """
        AUTHENTICATED
        Renews the VM in the selected room

        :param room_code: Room code to renew the VM in
        """
        if not self.authenticated:
            raise Exception("Not authenticated")

        csrf_token = fetch_pattern(self.session, f'/room/{room_code}', 'csrf-script')

        return http_post(
            self.session,
            '/api/vm/renew',
            data={'code': room_code},
            headers={'csrf-token': csrf_token},
            has_success=True
        )['timeInSeconds']

    def instance_terminate(self, room_code):
        """
        AUTHENTICATED
        Terminates the VM in the selected room

        :param room_code: Room code to terminate the VM in
        """
        if not self.authenticated:
            raise Exception("Not authenticated")

        csrf_token = fetch_pattern(self.session, f'/room/{room_code}', 'csrf-script')

        return http_post(
            self.session,
            '/api/vm/terminate',
            data={'code': room_code},
            headers={'csrf-token': csrf_token},
            has_success=True
        )



import requests
import re

__root_url = 'https://tryhackme.com'
__csrf_script_regex = re.compile("const csrfToken='(.{36})';")
__csrf_input_regex = re.compile('<input type="hidden" name="_csrf" value="(.{36})">')


def http_get(session, path, res_format="json", has_success=False, headers={}):
    """
    HTTP GET call wrapper

    :type session: requests.Session
    :param session: HTTP Session
    :type path: string
    :param path: HTTP Route
    :type res_format: string
    :param res_format: Response format
    :type has_success: bool
    :param has_success: Does the request have a "success" field
    :type headers: dict
    :param headers: Additional headers
    :return: Return object
    """

    try:
        r = session.get(f'{__root_url}{path}', headers=headers)

        if r.status_code == 200:
            if res_format == 'json':
                if has_success:
                    res = r.json()

                    if res['success']:
                        return res
                    else:
                        if 'message' in res:
                            raise Exception(f'API Error: {res["message"]}')
                        else:
                            raise Exception(f'API Error (no reason given)\nResponse: {res}')
                else:
                    return r.json()
            else:
                return r.content.decode('utf-8')
        else:
            raise Exception('HTTP Response was not 200')
    except Exception as err:
        raise err


def fetch_pattern(session, path, pattern):
    """
    Fetches a pattern from a raw HTTP response

    :param session: Session
    :param path: HTTP Path
    :param pattern: Compiled regular expression ID (csrf-input/csrf-script)
    :return: String matching group 1
    """

    r = http_get(session, path, res_format='')

    if pattern == 'csrf-input':
        return __csrf_input_regex.search(r).group(1)
    elif pattern == 'csrf-script':
        return __csrf_script_regex.search(r).group(1)
    else:
        return ""


def http_post(session, path, data, res_format="json", has_success=False, headers={}):
    """
    HTTP POST call wrapper

    :type session: requests.Session
    :param session: HTTP Session
    :type path: string
    :param path: HTTP Route
    :type data: dict
    :param data: HTTP Data (JSON)
    :type res_format: string
    :param res_format: Response format
    :type has_success: bool
    :param has_success: Does the request have a "success" field
    :type headers: dict
    :param headers: Additional headers
    :return: Return object
    """

    try:
        r = session.post(f'{__root_url}{path}', json=data, headers=headers)

        if r.status_code == 200:
            if res_format == 'json':
                if has_success:
                    res = r.json()

                    if res['success']:
                        return res
                    else:
                        if 'message' in res:
                            raise Exception(f'API Error: {res["message"]}')
                        else:
                            raise Exception(f'API Error (no reason given)\nResponse: {res}')
                else:
                    return r.json()
            else:
                return r.content.decode('utf-8')
        else:
            raise Exception(f'HTTP Response was not 200 (got {r.status_code})')
    except Exception as err:
        raise err



from .util import *


class __THMAuth(object):
    def login(self, credentials):
        """
        Log in to THM

        :param credentials: Login and password
        :return: null
        """

        csrf_token = fetch_pattern(self.session, '/login', 'csrf-input')

        data = {
            'email': credentials['username'],
            'password': credentials['password'],
            '_csrf': csrf_token
        }

        http_post(self.session, '/login', data, res_format='')

        try:
            test_request = http_get(self.session, '/message/get-unseen')

            if test_request['success']:
                self.authenticated = True
                return
            else:
                raise Exception(f'Failed to authenticate')
        except Exception as e:
            raise e
